/* Open Source License

Copyright 2005 by Frank McCown
All rights reserved.

Developed by:  Frank McCown
               Old Dominion University
               http://www.cs.odu.edu/~fmccown

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal with the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

         - Redistributions of source code must retain the above copyright
           notice, this list of conditions and the following disclaimers.
         - Redistributions in binary form must reproduce the above copyright
           notice, this list of conditions and the following disclaimers in the
           documentation and/or other materials provided with the distribution.
         - Neither the names of Open Archives Initiative Metadata Harvesting
           Project, Old Dominion University, nor the names of
           its contributors may be used to endorse or promote products derived
           from this Software without specific prior written permission.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS WITH THE SOFTWARE.

*/

using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;

namespace oaidemo
{
	/// <summary>
	/// Summary description for GetRecord.
	/// </summary>
	public class GetRecord : System.Web.UI.Page
	{
		protected string errorMessage = null;
		protected string metadataPrefix = null;
		protected string identifier = null;
		protected Record singleRec;

		protected struct Record
		{
			public string Header;
			public string Metadata;
		}

		private void Page_Load(object sender, System.EventArgs e)
		{
			// GetRecord MUST have identifier AND metadataPrefix

			int numKeys = Request.QueryString.Count + Request.Form.Count;
			if (numKeys > 3)
			{
				errorMessage = @"<error code=""badArgument"">Only the required arguments 'identifier' and 'metadataPrefix' may be given.</error>";
			}

			// Check for identifier
			identifier = Request["identifier"];
			if (identifier == null)
			{
				// Missing identifier
				errorMessage += @"<error code=""badArgument"">Required argument 'identifier' is missing.</error>";
			}
			else if (identifier.Length == 0)
			{
				errorMessage += @"<error code=""badArgument"">Content of argument 'identifier' is missing.</error>";
			}
			else if (!Util.CheckId(identifier))
			{
				errorMessage += @"<error code=""badArgument"">Identifier '" + identifier + "' is not a valid URI.</error>";
			}


			// Check for metadataPrefix
			metadataPrefix = Request["metadataPrefix"];
			if (metadataPrefix == null)
			{
				// Missing metadataPrefix
				errorMessage += @"<error code=""badArgument"">Required argument 'metadataPrefix' is missing.</error>";
			}
			else if (metadataPrefix.Length == 0)
			{
				errorMessage += @"<error code=""badArgument"">Content of argument 'metadataPrefix' is missing.</error>";
			}
			else if (!CheckMetadataPrefix(metadataPrefix))
			{
				errorMessage += @"<error code=""cannotDisseminateFormat"">'" + metadataPrefix + "' is not a valid metadataPrefix.</error>";
			}


			if (errorMessage == null)
			{
				// Must have valid identifier and metadataPrefix

				GetSingleRecord();
			}
		}

		private void GetSingleRecord()
		{
			string scheme, namespaceIdentifier, localIdentifier;
			bool ret = Util.ParseOAIIdentifier(identifier, out scheme, out namespaceIdentifier, out localIdentifier);
            
			if (!ret || (scheme != "oai") || (namespaceIdentifier != (string)Application["NamespaceIdentifier"]))
			{
				errorMessage = @"<error code=""idDoesNotExist"">identifier is not valid.</error>";
				return;
			}

            DataSet ds = (DataSet)Application["RecordsDataSet"];

			DataRow[] rows = ds.Tables["record"].Select("id = '" + localIdentifier + "'");
			if (rows.Length == 0)
			{
				errorMessage = @"<error code=""idDoesNotExist"">identifier '" + Server.HtmlEncode(identifier) + "' not found.</error>";
				return;
			}

			// Store for use by metadata handler
			Session["Row"] = rows[0];

			string date = Util.GetDate(rows[0]["lastModDate"].ToString());

			singleRec = new Record();
			singleRec.Header = "<identifier>" + identifier + "</identifier>\n" +
				"\t\t\t<datestamp>" + date + "</datestamp>\n"; 

			// Sets are not supported, so don't worry about <setSpec> tags

			// Execute the metadata format handler for the given metadataPrefix
			string metadataHandler = null;
			ArrayList acceptFormats = (ArrayList)Application["AcceptFormats"];
			foreach (Global.AcceptFormats af in acceptFormats)
			{
				if (af.Format == metadataPrefix)
				{
					metadataHandler = af.Handler;					
					break;
				}
			}

			if (metadataHandler == null)
			{
				// resumptionToken must have been altered and contains unsupported metadata format
				errorMessage = @"<error code=""cannotDisseminateFormat"">'" + metadataPrefix + "' is not a valid metadataPrefix.</error>";
				return;
			}
			else
			{
				System.IO.StringWriter writer = new System.IO.StringWriter();
				Server.Execute(metadataHandler, writer);
				singleRec.Metadata = writer.ToString();
			}

		}

		/// <summary>
		/// Checks to see if given metadataPrefix is supported.
		/// </summary>
		/// <param name="prefix"></param>
		/// <returns>Returns true if metadataPrefix is supported, false otherwise.</returns>
		private bool CheckMetadataPrefix(string prefix)
		{
			ArrayList acceptFormats = (ArrayList)Application["AcceptFormats"];
			foreach (Global.AcceptFormats af in acceptFormats)
			{
				if (af.Format == prefix)
					return true;
			}
			return false;
		}
	

		#region Web Form Designer generated code
		override protected void OnInit(EventArgs e)
		{
			//
			// CODEGEN: This call is required by the ASP.NET Web Form Designer.
			//
			InitializeComponent();
			base.OnInit(e);
		}
		
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{    
			this.Load += new System.EventHandler(this.Page_Load);

		}
		#endregion
	}
}
